@@ -125,7 +125,9 @@ def infer_constructor_params(
 infer_params = infer_constructor_params  # Legacy name
 
 
-def infer_method_params(cls: Type[T], method: Callable) -> Dict[str, inspect.Parameter]:
+def infer_method_params(
+    cls: Type[T], method: Callable, infer_kwargs: bool = True
+) -> Dict[str, inspect.Parameter]:
     signature = inspect.signature(method)
     parameters = dict(signature.parameters)
 
@@ -140,7 +142,7 @@ def infer_method_params(cls: Type[T], method: Callable) -> Dict[str, inspect.Par
     if var_positional_key:
         del parameters[var_positional_key]
 
-    if not has_kwargs:
+    if not has_kwargs or not infer_kwargs:
         return parameters
 
     # "mro" is "method resolution order". The first one is the current class, the next is the
@@ -301,30 +303,12 @@ def construct_arg(
     popped_params: Params,
     annotation: Type,
     default: Any,
-    could_be_step: bool = True,
     **extras,
 ) -> Any:
     """
     The first two parameters here are only used for logging if we encounter an error.
     """
-    from tango.step import Step, _RefStep
-
-    if could_be_step:
-        # We try parsing as a step _first_. Parsing as a non-step always succeeds, because
-        # it will fall back to returning a dict. So we can't try parsing as a non-step first.
-        backup_params = deepcopy(popped_params)
-        try:
-            return construct_arg(
-                class_name,
-                argument_name,
-                popped_params,
-                Step[annotation],  # type: ignore
-                default,
-                could_be_step=False,
-                **extras,
-            )
-        except (ValueError, TypeError, ConfigurationError, AttributeError):
-            popped_params = backup_params
+    from tango.step import Step
 
     origin = getattr(annotation, "__origin__", None)
     args = getattr(annotation, "__args__", [])
@@ -351,16 +335,9 @@ def construct_arg(
                     popped_params = Params({"type": popped_params})
             elif isinstance(popped_params, dict):
                 popped_params = Params(popped_params)
-            result = annotation.from_params(params=popped_params, **subextras)
+            result = annotation.from_params(popped_params, **subextras)
 
             if isinstance(result, Step):
-                if isinstance(result, _RefStep):
-                    existing_steps: Dict[str, Step] = extras.get("existing_steps", {})
-                    try:
-                        result = existing_steps[result.ref()]
-                    except KeyError:
-                        raise _RefStep.MissingStepError(result.ref())
-
                 expected_return_type = args[0]
                 return_type = inspect.signature(result.run).return_annotation
                 if return_type == inspect.Signature.empty:
@@ -369,11 +346,15 @@ def construct_arg(
                         "debugging, so we recommend them highly.",
                         result.__class__.__name__,
                     )
-                elif not issubclass(return_type, expected_return_type):
-                    raise ConfigurationError(
-                        f"Step {result.name} returns {return_type}, but "
-                        f"we expected {expected_return_type}."
-                    )
+                else:
+                    try:
+                        if not issubclass(return_type, expected_return_type):
+                            raise ConfigurationError(
+                                f"Step {result.name} returns {return_type}, but "
+                                f"we expected {expected_return_type}."
+                            )
+                    except TypeError:
+                        pass
 
             return result
         elif not optional:
@@ -394,7 +375,9 @@ def construct_arg(
         if type(popped_params) == str or isinstance(popped_params, Path):
             return str(popped_params)  # type: ignore
         else:
-            raise TypeError(f"Expected {argument_name} to be a string.")
+            raise TypeError(
+                f"Expected {argument_name} to be a string, found {popped_params} ({type(popped_params)})"
+            )
     elif annotation == float:
         # Floats are special because in Python, you can put an int wherever you can put a float.
         # https://mypy.readthedocs.io/en/stable/duck_type_compatibility.html
@@ -545,7 +528,7 @@ class FromParams(CustomDetHash):
     @classmethod
     def from_params(
         cls: Type[T],
-        params: Union[Params, dict, str],
+        params_: Union[Params, dict, str],
         constructor_to_call: Callable[..., T] = None,
         constructor_to_inspect: Union[Callable[..., T], Callable[[T], None]] = None,
         **extras,
@@ -575,6 +558,8 @@ class FromParams(CustomDetHash):
 
         from tango.common.registrable import Registrable  # import here to avoid circular imports
 
+        params = params_
+
         logger.debug(
             f"instantiating class {cls} from params {getattr(params, 'params', params)} "
             f"and extras {set(extras.keys())}"
@@ -615,6 +600,8 @@ class FromParams(CustomDetHash):
 
             as_registrable = cast(Type[Registrable], cls)
             default_to_first_choice = as_registrable.default_implementation is not None
+            if "type" in params and params["type"] not in as_registrable.list_available():
+                as_registrable.search_modules(params["type"])
             choice = params.pop_choice(
                 "type",
                 choices=as_registrable.list_available(),
@@ -635,7 +622,7 @@ class FromParams(CustomDetHash):
                 # mypy can't follow the typing redirection that we do, so we explicitly cast here.
                 retyped_subclass = cast(Type[T], subclass)
                 return retyped_subclass.from_params(
-                    params=params,
+                    params,
                     constructor_to_call=constructor_to_call,
                     constructor_to_inspect=constructor_to_inspect,
                     **extras,
@@ -647,7 +634,7 @@ class FromParams(CustomDetHash):
                 # instead of adding a `from_params` method for them somehow.  We just trust that
                 # you've done the right thing in passing your parameters, and nothing else needs to
                 # be recursively constructed.
-                return subclass(**params)  # type: ignore
+                return constructor_to_call(**params, **create_extras(constructor_to_call, extras))  # type: ignore
         else:
             # This is not a base class, so convert our params and extras into a dict of kwargs.
 
@@ -701,7 +688,7 @@ class FromParams(CustomDetHash):
         You don't need to implement this all the time. Tango will let you know if you
         need it.
         """
-        raise NotImplementedError()
+        raise NotImplementedError(f"{self.__class__.__name__}._to_params() needs to be implemented")
 
     def det_hash_object(self) -> Any:
         return self.to_params()
